"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _find = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

const async = require('async');

const shelljs = require('shelljs');

const fs = require('fs');

const path = require('path');

function compileSolcContract(logger, compileSettings, allowedDirectories, callback) {
  const command = `solc --standard-json --allow-paths ${allowedDirectories.join(',')}`;
  shelljs.ShellString((0, _stringify.default)(compileSettings)).exec(command, {
    silent: true
  }, (code, stdout, stderr) => {
    if (stderr) {
      logger.warn(stderr);
    }

    if (code !== 0) {
      return callback(`solc exited with error code ${code}`);
    }

    if (!stdout) {
      return callback('solc execution returned nothing');
    }

    callback(null, stdout.replace(/\n/g, ''));
  });
}

function getSolcVersion(logger, callback) {
  shelljs.exec('solc --version', {
    silent: true
  }, (code, stdout, stderr) => {
    if (stderr) {
      logger.warn(stderr);
    }

    if (code !== 0) {
      return callback(`solc exited with error code ${code}`);
    }

    if (!stdout) {
      return callback('solc execution returned nothing');
    }

    const result = stdout.match(/(\d+.\d+.\d+)/);
    callback(null, result[1]);
  });
}

function compileSolc(embark, contractFiles, contractDirectories, options, callback) {
  if (!contractFiles || !contractFiles.length) {
    return callback();
  }

  const logger = embark.logger;
  const outputBinary = embark.pluginConfig.outputBinary;
  const outputDir = embark.config.buildDir + embark.config.contractDirectories[0];
  const solcConfig = embark.config.embarkConfig.options.solc;
  let allowedDirectories = [];
  const remappings = [];
  const compilationSettings = {
    language: 'Solidity',
    sources: {},
    settings: {
      optimizer: {
        enabled: solcConfig['optimize'],
        runs: solcConfig['optimize-runs']
      },
      remappings,
      outputSelection: {
        '*': {
          '': ['ast'],
          '*': ['abi', 'devdoc', 'evm.bytecode', 'evm.deployedBytecode', 'evm.gasEstimates', 'evm.legacyAssembly', 'evm.methodIdentifiers', 'metadata', 'userdoc']
        }
      }
    }
  };
  async.waterfall([function checkSolc(next) {
    const solc = shelljs.which('solc');

    if (!solc) {
      logger.error('solc is not installed on your machine');
      logger.info('You can install it by following the instructions on: http://solidity.readthedocs.io/en/latest/installing-solidity.html');
      return next('Compiler not installed');
    }

    logger.info("compiling solidity contracts with command line solc...");
    next();
  }, function getContentAndRemappings(next) {
    async.each(contractFiles, (file, eachCb) => {
      file.prepareForCompilation(options.isCoverage).then(content => {
        var _context;

        // add contract directory and all it's recusrive import direcotries to allowed directories
        let dir = path.dirname(file.path);
        if (!(0, _includes.default)(allowedDirectories).call(allowedDirectories, dir)) allowedDirectories.push(dir);
        (0, _forEach.default)(_context = file.importRemappings).call(_context, importRemapping => {
          dir = path.dirname(importRemapping.target);
          if (!(0, _includes.default)(allowedDirectories).call(allowedDirectories, dir)) allowedDirectories.push(dir);
          const remapping = `${importRemapping.prefix}=${importRemapping.target}`;

          if (!(0, _includes.default)(remappings).call(remappings, remapping)) {
            remappings.push(remapping);
          }
        }); // TODO change this to Embark's utils function once embark-solc is in the mono-repo

        compilationSettings.sources[file.path.replace(/\\/g, '/')] = {
          content: content.replace(/\r\n/g, '\n')
        };
        eachCb();
      }).catch(eachCb);
    }, next);
  }, function compile(next) {
    compileSolcContract(logger, compilationSettings, allowedDirectories, (err, compileString) => {
      if (err) {
        return next(err);
      }

      let json;

      try {
        json = JSON.parse(compileString);
      } catch (e) {
        logger.error(e.message || e);
        return callback(`Compiling returned an unreadable result`);
      }

      embark.events.emit('contracts:compiled:solc', json);
      const contracts = json.contracts; // Check for errors

      if (json.errors) {
        var _context2;

        let isError = false;
        (0, _forEach.default)(_context2 = json.errors).call(_context2, error => {
          if (error.severity === 'error') {
            isError = true;
            logger.error(error.formattedMessage);
          } else {
            logger.warn(error.formattedMessage);
          }

          logger.debug(error.message); // Print more error information in debug
        });

        if (isError) {
          return next(`Error while compiling`);
        }
      }

      next(null, contracts);
    });
  }, function populateCompiledObject(contracts, next) {
    const compiledObject = {};

    for (let contractFile in contracts) {
      for (let contractName in contracts[contractFile]) {
        var _context3, _context4, _context5;

        let contract = contracts[contractFile][contractName];
        let filename = contractFile;

        for (let directory of contractDirectories) {
          let match = new RegExp("^" + directory);
          filename = filename.replace(match, '');
        }

        let className = contractName;
        const contractFileMatch = className.match(/.sol:(.*)/);

        if (contractFileMatch) {
          className = contractFileMatch[1];
        }

        compiledObject[className] = {};
        compiledObject[className].code = contract.evm.bytecode.object;
        compiledObject[className].linkReferences = contract.evm.bytecode.linkReferences;
        compiledObject[className].runtimeBytecode = contract.evm.deployedBytecode.object;
        compiledObject[className].realRuntimeBytecode = (0, _slice.default)(_context3 = contract.evm.deployedBytecode.object).call(_context3, 0, -68);
        compiledObject[className].swarmHash = (0, _slice.default)(_context4 = (0, _slice.default)(_context5 = contract.evm.deployedBytecode.object).call(_context5, -68)).call(_context4, 0, 64);
        compiledObject[className].gasEstimates = contract.evm.gasEstimates;
        compiledObject[className].functionHashes = contract.evm.methodIdentifiers;
        compiledObject[className].abiDefinition = contract.abi;
        compiledObject[className].userdoc = contract.userdoc;
        compiledObject[className].filename = filename;
        const normalized = path.normalize(filename);
        const origContract = (0, _find.default)(contractFiles).call(contractFiles, contractFile => (0, _includes.default)(normalized).call(normalized, path.normalize(contractFile.originalPath)));

        if (origContract) {
          compiledObject[className].originalFilename = path.normalize(origContract.originalPath);
        }
      }
    }

    next(null, compiledObject);
  }], (err, compiledObject) => {
    callback(err, compiledObject);

    if (outputBinary) {
      embark.events.once("outputDone", () => {
        async.eachOf(compiledObject, (contract, className, eachCb) => {
          fs.writeFile(path.join(outputDir, className + ".bin"), compiledObject[className].code, eachCb);
        }, err => {
          if (err) {
            logger.error("Error writing binary file", err.message || err);
          }
        });
      });
    }
  });
}

module.exports = {
  compileSolc,
  compileSolcContract,
  getSolcVersion
};
//# sourceMappingURL=Compiler.js.map